require 'incl_gametime'

-- the delay between spawn attempts, can be overridden by adding "spawnDelay" objvar
DEFAULT_DELAY_SECS = 60
-- the delay between spawn attempts, can be overridden by adding "spawnCount" objvar
DEFAULT_SPAWN_COUNT = 1
DEFAULT_SPAWN_RANGE = 0
DEFAULT_SPAWN_STATE = true

--[[ Counts the amount of players in range,
but excludes any got chars
]]--
function CountPlayers(players)
	local count = 0

	for i, player in pairs(players) do
		if (not player:IsGod()) then
			count = count + 1
		end
	end

	return count
end

--[[ This function handles the main work. It is called periodically and Checks
the amount of spawned mobs. If required the spawner creates new mobs
]]--

function CheckSpawn()
    local spawnData = this:GetObjVar("spawnData")

	local isActive = DEFAULT_SPAWN_STATE

	if (this:HasObjVar("Active")) then
		isActive = this:GetObjVar("Active")
	end

	local timeMatch = true -- currently for Nightspawn only, other options are planned

    if (this:HasObjVar("NightSpawn")) then
        DebugMessage("NightSpawn is " .. tostring(this:GetObjVar("NightSpawn")))
        if (IsDayTime()) then
            DebugMessage("time is " .. tostring(GetGameTimeOfDay()))
            local nearbyPlayers = FindObjects(SearchPlayerInRange(15))
            DebugMessage("nearbyPlayers is " ..tostring(#nearbyPlayers) .. " nearbyPlayers " .. tostring(nearbyPlayers))
            if (CountPlayers(nearbyPlayers) == 0) then
                this:DebugMessage("not nearby")
				Despawn()
            end

		timeMatch = false
        end
    end

	local spawnCount = this:GetObjVar("spawnCount") or DEFAULT_SPAWN_COUNT
    local templateId = this:GetObjVar("spawnTemplate")
    if( templateId == nil ) then
        DebugMessage("[Better_mob_spawner::CheckSpawn] No template id specified.")
        return
    end

    -- NOTE: If spawn count is made smaller, we are not destroying the extra mobs
    if spawnData == nil then
        spawnData = {}
        this:SetObjVar("spawnData", spawnData)
    elseif #spawnData > spawnCount then
    	for i=spawnCount+1,#spawnData do
            table.remove(spawnData,i)
        end
    end

    --DebugMessage("---CheckSpawn SpawnCount: "..spawnCount)

	if (isActive == true) then
		if (timeMatch == true) then
			local spawnRange = this:GetObjVar("spawnRange") or DEFAULT_SPAWN_RANGE

			for i=1, spawnCount do
				--DebugMessage("---Checking i="..i)
				if( spawnData[i] == nil or not(spawnData[i]:IsValid()) ) then
					--DebugMessage("---Create "..templateId)
					-- only create one at a time
					local location = this:GetLoc()

					DebugMessage("---CheckSpawn spawnRange: "..spawnRange)

					if (spawnRange > 0) then
						location = this:GetLoc():Project(math.random(0,360), math.random(2,spawnRange))
					end

					CreateObj(templateId, location, "mobSpawned", i)
					break
				end
			end
		end

		if (not this:HasTimer("spawnTimer")) then
			local spawnDelay = this:GetObjVar("spawnDelay") or DEFAULT_DELAY_SECS
			this:ScheduleTimerDelay(TimeSpan.FromSeconds(spawnDelay), "spawnTimer")
		end
	end

	UpdateSpawnerTooltipString()
end


--[[
Despawns all currently spawned mobs
]]--
function Despawn()
	local spawnData = this:GetObjVar("spawnData")

	if (spawnData ~= nil) then
		for i,j in pairs(spawnData) do
			j:Destroy()
			--DebugMessage("Destroying")
			table.remove(spawnData,i)
		end

		this:SetObjVar("spawnData",spawnData)
	end
end

function UpdateSpawnerTooltipString()
	local spawnCount = this:GetObjVar("spawnCount") or DEFAULT_SPAWN_COUNT
	local currentSpawnCount = 0
    local templateId = this:GetObjVar("spawnTemplate") or "-nil-"
	local spawnDelay = this:GetObjVar("spawnDelay") or DEFAULT_DELAY_SECS
	local spawnData = this:GetObjVar("spawnData")
	local nightSpawner = this:HasObjVar("NightSpawn")
	local isActive = DEFAULT_SPAWN_STATE

	if (this:HasObjVar("Active")) then
		isActive = this:GetObjVar("Active")
	end

	if (spawnData ~= nil) then
		currentSpawnCount = #spawnData
	end

	local tooltipString = ""

	if (isActive == true) then
		tooltipString = "[00FF00]" .. "(Active)" .. "[-]\n"
	else
		tooltipString = "[B3B1E6]" .."(Inactive)".. "[-]\n"
	end

	if (nightSpawner == true) then
		tooltipString = tooltipString.."(Night Spawner)"

		if (IsDayTime() == true) then
			tooltipString = tooltipString.." (not in range)\n"
		else
			tooltipString = tooltipString.." (in range)\n"
		end
	end

	tooltipString = tooltipString .. "Count: "..currentSpawnCount.."/"..spawnCount.."\nTemplate: "..templateId.."\nDelay: "..spawnDelay
	this:SetSharedObjectProperty("TooltipString", tooltipString)

	if (not this:HasTimer("UpdateTooltip")) then
		this:ScheduleTimerDelay(TimeSpan.FromSeconds(20),"UpdateTooltip")
	end
end


--[[
Displays the Spawner Gump to the players
]]--
function DisplayGump(user)

	local spawnCount = this:GetObjVar("spawnCount") or DEFAULT_SPAWN_COUNT
	local currentSpawnCount = 0
    local templateId = this:GetObjVar("spawnTemplate") or "-nil-"
	local spawnDelay = this:GetObjVar("spawnDelay") or DEFAULT_DELAY_SECS
	local spawnData = this:GetObjVar("spawnData")
	local isActive = DEFAULT_SPAWN_STATE

	if (this:HasObjVar("Active")) then
		isActive = this:GetObjVar("Active")
	end

	UpdateSpawnerTooltipString()

	if (spawnData ~= nil) then
		currentSpawnCount = #spawnData
	end

	local newWindow = DynamicWindow("SpawnerGump","Spawner Entries",700,440)

	newWindow:AddLabel(20, 20, "Spawn Count: "..currentSpawnCount.."/"..spawnCount)
	newWindow:AddLabel(20, 40, "Template: "..templateId)
	newWindow:AddLabel(20, 60, "Spawn Delay: "..spawnDelay)


	newWindow:AddLabel(20, 80, "Spawned:")

	if (spawnData ~= nil) then

	local index = 0;

	for id,entry in ipairs(spawnData) do
			local template = tostring(entry:GetCreationTemplateId())
			newWindow:AddLabel(20, 100+(20*index), template)
			index = index +1
		end
	end

	local stateText = "[00FF00]Active[-]"

	if (isActive == false) then
		stateText = "[FF0000]Inactive[-]"
	end

	newWindow:AddButton(550, 20, "Status", stateText,0, 0)

	newWindow:AddButton(570, 380, "Home", "Home",0, 0)
	newWindow:AddButton(480, 380, "Respawn", "Spawn", 0, 0)
	newWindow:AddButton(340, 380, "Count", "Spawn Count", 0, 0)
	newWindow:AddButton(200, 380, "Range", "Spawn Range", 0, 0)
	newWindow:AddButton(90, 380, "Template", "Template", 0, 0)
	newWindow:AddButton(10, 380, "Delay", "Delay", 0, 0)

	user:OpenDynamicWindow(newWindow, this)

end

--[[
This message handler processes the respawn command. All Spawn data will be erased
and a full respawn will be done
]]--
RegisterEventHandler(EventType.Message, "Respawn",
	function (user)
		DebugMessage("Spawner: Handle Respawn Message")

		if (user == nil) then return end
			Despawn()

			CheckSpawn()
			user:SystemMessage("Respawn Done!")
			UpdateSpawnerTooltipString()
	end)

--[[
This message handler moves all spawned items back home
]]--
RegisterEventHandler(EventType.Message, "BringHome",
	function (user)
		DebugMessage("Spawner: Handle Respawn Message")

		if (user == nil) then return end
			local spawnData = this:GetObjVar("spawnData")

		if (spawnData ~= nil) then
				for i,j in pairs(spawnData) do
					if (j:IsValid()) then
						j:SetWorldPosition(this:GetLoc())
					end
				end
			end
		user:SystemMessage("Brought objects back home!")
		UpdateSpawnerTooltipString()
end)

--[[
This message handles the spawner state
]]--
RegisterEventHandler(EventType.Message, "ToggleSpawnerStatus",
	function (user)
		if (user == nil) then return end

		local isActive = DEFAULT_SPAWN_STATE

		if (this:HasObjVar("Active")) then
			isActive = this:GetObjVar("Active")
		end

		isActive = (not isActive)

		DebugMessage("Spawner: Handle Spawner Status:"..tostring(isActive))
		this:SetObjVar("Active", isActive)

		DebugMessage("Spawner: Spawner Status now:"..tostring(isActive))

		UpdateSpawnerTooltipString()

		if (isActive and not this:HasTimer("spawnTimer")) then
			local spawnDelay = this:GetObjVar("spawnDelay") or DEFAULT_DELAY_SECS
			this:ScheduleTimerDelay(TimeSpan.FromSeconds(spawnDelay), "spawnTimer")
		end
end)


--[[
Eventhandler for the spawner gump
]]--

RegisterEventHandler(EventType.DynamicWindowResponse, "SpawnerGump",
function (user,buttonId)

		if (buttonId == "") then
			return
		end

	if (buttonId:match("Respawn") ) then
		DebugMessage("Spawner: Respawn")
		this:SendMessage("Respawn", user)
	end

	if (buttonId:match("Home") ) then
		DebugMessage("Spawner: Bring Home")
		this:SendMessage("BringHome", user)
	end

	if (buttonId:match("Status") ) then
		DebugMessage("Spawner: Changing Status")
		this:SendMessage("ToggleSpawnerStatus", user)
	end

	if (buttonId:match("Count") ) then
		user:SendClientMessage("EnterChat","/setobjvar".." "..this.Id.." spawnCount".." ")
	end

	if (buttonId:match("Range") ) then
		user:SendClientMessage("EnterChat","/setobjvar".." "..this.Id.." spawnRange".." ")
	end

	if (buttonId:match("Template") ) then
		user:SendClientMessage("EnterChat","/setobjvar".." "..this.Id.." spawnTemplate".." ")
	end

	if (buttonId:match("Delay") ) then
		user:SendClientMessage("EnterChat","/setobjvar".." "..this.Id.." spawnDelay".." ")
	end

	UpdateSpawnerTooltipString()
end)

--[[
This event handler respawns the objects after it is attached
]]--
RegisterEventHandler(EventType.ModuleAttached, "better_mob_spawner",
	function()
		CheckSpawn()
		UpdateSpawnerTooltipString()
	end)

--[[
This event handler can be used to modify the spawned object after it
has been moved to the world
]]--
RegisterEventHandler(EventType.CreatedObject, "mobSpawned",
	function(success, objref, index)
		if( success) then
			local spawnData = this:GetObjVar("spawnData")
			spawnData[index] = objref
			this:SetObjVar("spawnData",spawnData)
            --DebugMessage("Set facing to " .. tostring(this:GetFacing()))
	        objref:SetFacing(this:GetFacing())
		end
	end)

--[[
Main spawn timer. Checks the state after the given interval
]]--
RegisterEventHandler(EventType.Timer, "spawnTimer",
	function()
		CheckSpawn()
	end)

--[[
Used to display the gump to the player after doubleclicking the spawner
]]--
RegisterEventHandler(EventType.Message, "UseObject",
	function (user)
		DisplayGump(user)
	end)

--[[
Updates the tooltip in a regular interval
]]--
RegisterEventHandler(EventType.Message, "UpdateTooltip",
	function()
		UpdateSpawnerTooltipString()
	end)


