-- Internal structure of the party
-- the memberinfo table keeps track of the looting
-- decisions
PartyMemberInfo = {
					Create = function(m)
						local res = {}

						res.Mobile=m;
						res.CanLoot=true;
						return res;
					end
				  }

-- Creates a random guid as party Id
local random = math.random
local function uuid()
    local template ='xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'
    return string.gsub(template, '[xy]', function (c)
        local v = (c == 'x') and random(0, 0xf) or random(8, 0xb)
        return string.format('%x', v)
    end)
end


-- Main Party Module
Party = {
		Capacity = 10,

		InviteTarget = function()
			this:RequestClientTargetGameObj(this, "partyinvite")
		end,

		GetCandidates = function(p)
		  local count = 0
		  for _ in pairs(p.Candidates) do count = count + 1 end
		  return count
		end,

		GetMembers = function(p)
		 local count = 0
		  for _ in pairs(p.Members) do count = count + 1 end
		  return count
		end,

		Get = function (mobile)

			if (mobile == nil) then
				return nil;
			end

			return FindPartyForPlayer(mobile, partys)
		end,

		Invite = function ( from, target )
			local p = Party.Get( from );

			if ( p == nil ) then
				p = Party.Create( from );
			end

			if ( p.Candidates[target] == nil ) then
				p.Candidates[target] = target;
			end

			Party.SetPartyTitle(from, p)

			from:SetObjVar("Party",p.Id)

			target:SystemMessage("You are invited to join the party. Type /accept to join or /decline to decline the offer.")

			from:SystemMessage( "You have invited them to join the party." );

			target:SetObjVar("PartyInvitation", from)

			target:ScheduleTimerDelay(TimeSpan.FromSeconds(15.0),"DeclineTimer",target)

			GetPartyController():SetObjVar(p.Id, p)
		end,

		Decline = function()

		if( this:HasObjVar("PartyInvitation") ) then

			local partyInvitation = this:GetObjVar("PartyInvitation")

			if (partyInvitation == nil) then
				this:SystemMessage( "Decline: Nobody invited you" );
				return
			end

			local p = Party.Get(partyInvitation)

			if (p == nil) then
				this:SystemMessage( "The party no longer Exists" );
				this:SetObjVar("PartyInvitation",nil);
				return
			end

			p.Leader:SystemMessage( "Does not wish to join the party." );

			this:SystemMessage( "You notify them that you do not wish to join the party." );

			this:SetObjVar("PartyInvitation",nil);

			p.Candidates[this]=nil;
			DebugMessage("cleared Candidate");

			DebugMessage("num candidates:" .. Party.GetCandidates(p));

			if ( Party.GetCandidates(p) == 0 and Party.GetMembers(p) <= 1 ) then
				p.Leader:SystemMessage("Your party has been disbanded");
				p.Leader:SetObjVar("Party", nil);
				Party.ResetPartyTitle(p.Leader)

				GetPartyController():DelObjVar(p.Id)

			else
				GetPartyController():SetObjVar(p.Id, p)
			end
		end
	end,

	Accept = function()
			if( this:HasObjVar("PartyInvitation") ) then
				local partyInvitation = this:GetObjVar("PartyInvitation")

				this:RemoveTimer("DeclineTimer")

				local p = Party.Get(partyInvitation)

				if (p == nil) then
					this:SystemMessage( "The party no longer Exists" );
					this:SetObjVar("PartyInvitation",nil);
					return
				end

				this:SetObjVar("PartyInvitation",nil);

				--p.Leader:SendClientMessage("PlayLocalEffect",{this,"ChargedWeaponEffect",30})

				p.Candidates[this]=nil;

				local mi = p.Members[this];

				if ( mi == nil ) then
					p.Members[this] = PartyMemberInfo.Create( this );
					this:SetObjVar("Party", p.Id);
					Party.SetPartyTitle(this, p)

					Party.SendToAll( this, p, "joined the party.");

					this:SystemMessage( "You have been added to the party." );

				end

				GetPartyController():SetObjVar(p.Id, p)
			else
				this:SystemMessage( "Accept: Nobody invited you" );
			end
	end,

	Contains = function( m , p )
			return ( p.Members[m] ~= nil );
	end,

	Remove = function ( m, p )
		if ( m == p.Leader ) then

				for mobile, memberinfo in pairs(p.Members) do
					mobile:SystemMessage("Your party has been disbanded");
					mobile:SetObjVar("Party", nil)
					Party.ResetPartyTitle(mobile)
				end

				GetPartyController():DelObjVar(p.Id)
		else
				m:SystemMessage("You have been removed from the party.");

				Party.SendToAll( nil, p, "A player has been removed from your party.");

				if (p.Members[m] ~= nil) then
					p.Members[m] = nil;
					m:SetObjVar("Party", nil);
					Party.ResetPartyTitle(m);
				end

				if ( Party.GetMembers(p) == 1 ) then
					Party.SendToAll(nil, p, "The last person has left the party..." );

					GetPartyController():DelObjVar(p.Id)
					p.Leader:SetObjVar("Party", nil)
					Party.ResetPartyTitle(p.Leader)

				else
					GetPartyController():SetObjVar(p.Id, p)
				end
		end
	end,

	SendToAll = function(from, p , line)

		local name = ""
		if ( from ~= nil) then
			name = from:GetName()
		end

		for mobile, memberinfo in pairs(p.Members) do
			mobile:SystemMessage( " [Party]->" .. name ..":" .. line) ;
		end
	end,

	SendMessage = function (...)

	local p = Party.Get(this)

		if (p == nil) then
			return
		end

		local arg = table.pack(...)

		local line = ""
		if(#arg > 0) then
			for i = 1,#arg do line = line .. tostring(arg[i]) .. " " end
		end

		Party.SendToAll( this, p, line);
	end,

	Command = function(command)

		local array = {}

		local p = Party.Get(this)

		if (p ~= nil) then
			array=p.Members
		end

		local newWindow = DynamicWindow("PartyWindow","Party",600,440)

		local y = 10
		local itemsPerPage = 10
		local startIndex = (objVarPage-1) * itemsPerPage
		local i = 0
		for mobile,entry in pairs(array) do
			if( i >= startIndex and i <= startIndex + itemsPerPage) then
				local varName = mobile:GetName()
				if( varName:len() > 25 ) then
					varName = varName:sub(1,22).."..."
				end
				newWindow:AddLabel(20, y, varName)

				if (entry ~= nil) then

					if ( entry.CanLoot ) then
						newWindow:AddLabel(430, y, "(Lootable)")
					else
						newWindow:AddLabel(430, y, "(NOT Lootable)")
					end
				end

				if (this == p.Leader and mobile ~= this) then
					newWindow:AddButton(370, y, "Kick-"..mobile.Id, "X", 0, 0)
					--newWindow:AddButton(370, y, "Kick-"..entry.name, "View", 0, 0, "", "", false)
				end
				--local varType = type(entry.value)
				--if( varType == "userdata" or varType == "table") then
				--	newWindow:AddButton(300, y-5, "View-"..entry.name, "View", 0, 0, "", "", false)
				--else
					--newWindow:AddLabel(300, y, tostring(entry.value))
				--end
				y = y + 30
				i = i + 1
			end
		end
			newWindow:AddButton(20, 380, "Invite", "Invite", 0, 0)
			newWindow:AddButton(200, 380, "Loot", "Loot", 0, 0)
			newWindow:AddButton(100, 380, "Dump", "Dump", 0, 0)
			newWindow:AddButton(300, 380, "Refresh", "Refresh", 0, 0)
			newWindow:AddButton(480, 380, "Leave", "Leave", 0, 0)

			--window:AddButton(285,530,"Increase","More",245,0)

			this:OpenDynamicWindow(newWindow)
	end,
	Create = function(leader)
		local res = {}
		res.Leader = leader;

		res.Members = {}
		res.Candidates = {}

		res.Members[leader] = PartyMemberInfo.Create( leader );
		res.Id = uuid()
		leader:SystemMessage("You created a party");

		GetPartyController():SetObjVar(res.Id, res)

		return res
	end,
	SetPartyTitle = function (user, p)
		local partystring = "(Party: "..p.Leader:GetName()..")";
		local oldTitle = user:GetSharedObjectProperty("Title")

		if (oldTitle ~=partystring) then
			user:SetObjVar("OriginalTitle", oldTitle)
			user:SetSharedObjectProperty("Title", partystring)
		end
	end,
	ResetPartyTitle = function (user)
		local oldTitle = user:GetObjVar("OriginalTitle")
		if (oldTitle ~= nil) then
			user:SetSharedObjectProperty("Title", oldTitle)
			user:SetObjVar("OriginalTitle",nil)
		end
	end
    }
