-- This is a scriptlet host base
-- Scriptlets are Lua modules which contain many scripts
-- Example: ai_modular

-- To create new host, do the following:
-- 1. Require this file at beginning of file
-- 2. Set MODULES_OBJVAR to desired unique objvar name
-- 3. At end of your file, call RequireModules()

-- List of available modules (not necessarily everything required, but indicates if module is active and safe to use)
Modules = {}

-- Post requires
PostRequires = {}
-- Post tasks
PostTasks = {}

-- Adds new post require
function AddPostRequire(mod)
	PostRequires[#PostRequires+1] = mod
end

function AddPostTask(func)
	PostTasks[#PostTasks+1] = func
end

function RequireModules()
	local mods = this:GetObjVar(MODULES_OBJVAR) or {}

	for priority,name in ipairs(mods) do
		--DebugMessage("Loading module "..name)
		--LuaDebugCallStack()
		require(name)
		--DebugMessage("Done: "..this:GetName())
	end
	--DebugMessage("For loop done: "..this:GetName())

	ProcessPostRequires()
	ProcessPostTasks()
end

-- Process post requires (data files) provided by AI modules
function ProcessPostRequires()
	for i,mod in ipairs(PostRequires) do
		--DebugMessage("Post require: "..this:GetName())
		require(mod)
	end
end

function ProcessPostTasks()
	for i,func in ipairs(PostTasks) do
		func()
	end
end

-- Adds module and reloads this
function AddModule(name)
	local mods = this:GetObjVar(MODULES_OBJVAR)
	mods[#mods+1] = name
	this:SetObjVar("AIModules", mods) -- Since we set to this, it is syncronous
	--this:AddModule("ai_modular_reloader")
	ReloadModule(GetCurrentModule())
end

-- Deletes module and reloads this
function DelModule(name)
	local mods = this:GetObjVar(MODULES_OBJVAR)
	for i,v in pairs(mods) do
		if v == name then
			table.remove(mods, i)
			break
		end
	end
	this:SetObjVar(MODULES_OBJVAR, mods) -- Since we set to this, it is syncronous
	--this:AddModule("ai_modular_reloader")
	ReloadModule(GetCurrentModule())
end