-- Effect handlers can be used to abstract certain effect types for multiple purposes (TODO)

require "incl_effect_utils"

EffectHandlers = {}

EffectHandlers.StatModifier = {
	OnApply = function(self)
		for name,data in pairs(self.args) do
			if type(data) == "table" then
				self.gameObj:RequestAddStatMod(name, self.args.Identifier, data.Type, data.Value)
			end
		end
	end,

	OnRemove = function(self)
		for name,data in pairs(self.args) do
			if type(data) == "table" then
				self.gameObj:RequestRemoveStatMod(name, self.args.Identifier)
			end
		end
	end
}

EffectHandlers.Announce = {
	OnApply = function(self)
		if type(self.args.ApplyMessage) == "string" then
			self.gameObj:SystemMessage(self.args.ApplyMessage)
		end
	end,

	OnRemove = function(self)
		if type(self.args.RemoveMessage) == "string" then
			self.gameObj:SystemMessage(self.args.RemoveMessage)
		end
	end
}

EffectHandlers.ApplyEffect = {
	OnApply = function(self)
		local effects = self.args.Apply
		if type(effects) == "table" then
			for _,effect in pairs(effects) do
				InflictEffect(self.gameObj, effect)
			end
		end
	end,

	OnRemove = function(self)
		local effects = self.args.Remove
		if type(effects) == "table" then
			for _,effect in pairs(effects) do
				InflictEffect(self.gameObj, effect)
			end
		end
	end,

	OnTick = function(self)
		local effects = self.args.Tick
		if type(effects) == "table" then
			for _,effect in pairs(effects) do
				InflictEffect(self.gameObj, effect)
			end
		end
	end,

	OnWearOff = function(self)
		local effects = self.args.WearOff
		if type(effects) == "table" then
			for _,effect in pairs(effects) do
				InflictEffect(self.gameObj, effect)
			end
		end
	end
}

function GetEffectHandler(id)
	return EffectHandlers[id]
end

function ReadyEffectHandler(gameObj, args, handler)
	local env = {}
	env.gameObj = gameObj
	env.args = args
	for k,v in pairs(handler) do
		env[k] = v
	end

	return env
end

function ProduceEffectHandlers(gameObj,effect,event)
	local handlers = {}
	for k,v in pairs(effect) do
		if type(v) == "table" then
			local handler = GetEffectHandler(k)
			if handler ~= nil and type(handler[event]) == "function" then
				handlers[k] = ReadyEffectHandler(gameObj, v, handler)
			end
		end
	end

	return handlers
end