--- Extension to the default incl_gametime with functions for determining calendar dates
-- and allowing a fancy custom calendar

-- additional and modded functions
require 'incl_gametime_arcaniclock_timefuncs'



-- Overriding defaults
DEFAULT_DAYLIGHT_DURATION_SECS = 140 * 60
DEFAULT_NIGHTTIME_DURATION_SECS = 100 * 60
DEFAULT_DAYLIGHT_START_OFFSET = 6
DEFAULT_NIGHT_START_OFFSET = 22



--- Date in Realtime when the Game Calendar starts with game day 1/1/0001
-- This sets calendar start to 1/1/1800:==> myDate=DateTime.Parse("1/1/1800 0:0:0")
  local myDate=DateTime.Parse("1/1/1800 0:0:0")

-- Ticks in realtime since start of the in game calendar
GAMETIME_START=myDate.Ticks

-- How much is gametime compressed?
TIME_COMPRESSION_FACTOR=GetTotalDayDurationSecs()/(24*3600)



-------------------------------
--- SETTINGS
-------------------------------
-- INFO from .NET 4.0: 
-- Internally, all DateTime values are represented as the number of ticks 
-- (the number of 100-nanosecond intervals) that have elapsed since 
-- 12:00:00 midnight, January 1, 0001.
-- ---------------------------------------------------------------------------------

-------------------------------
--- Basic Datastructures
-------------------------------
MINUTE ={s="minute",p="minutes",seconds=60}
-- singular, plural how many seconds
HOUR={s="hour",p="hours",seconds=3600}
-- singular, plural how many hours
DAY={s="day",p="days",hours=24}

-- allow sections of a days, like "morning", teatime",
-- beginning hour ending hour
DAYSECTIONS={
{name="midnight",first=0, last=1},
{name="sleeptime",first=1, last=6},
{name="turnaround hour",first=6, last=7},
{name="eye rubbing hour",first=7, last=8},
{name="breakfast",first=8, last=9},
{name="morning work",first=9, last=10},
{name="second breakfast",first=10, last=11},
{name="late morning work",first=11, last=12},
{name="lunchbreak",first=12, last=13} ,
{name="nap time",first=13, last=14} ,
{name="afternoon work",first=14, last=16},
{name="teatime",first=16, last=17} ,
{name="late afternoon work",first=17, last=18},
{name="call it a day hour",first=18, last=19} ,
{name="dinnertime",first=19, last=20} ,
{name="snuggle hour",first=20, last=22},
{name="bedtime",first=22, last=24} 
}

-- MOONPHASES names
MOON={
"New moon",
"Waxing crescent",
"First quarter",
"Waxing gibbous",
"Full moon",
"Waning gibbous",
"Last quarter",
"Waning crescent"
}

-- Duration of a full moon month
MOONPHASE=29

--[[
HOURNAMES={
	1"hour of the midnight owl",
	2"hour of the ",
	3"",
	4"",
	5"",
	6"",
	7"",
	8"",
	9"",
	10"",
	11"",
	12"",
	13"hour of fire",
	14"",
	15"",
	16"",
	17"",
	18"Hour of Childrens",
	19"",
	20"",
	21"",
	22"",
	23""
}
]]--

DAYS={
	{name="Moonday"		},  -- WEEK[1]
	{name="Flowerday"	},  -- WEEK[2]
	{name="Weatherday"	},  -- WEEK[3]
	{name="Thunderday"	},  -- WEEK[4]
	{name="Fireday"		},  -- WEEK[5]
	{name="Waterday"	},  -- WEEK[6]
	{name="Sunday"		},  -- WEEK[7]
}


-- Start in the order of months
MONTHS={
	{name="Janury", 	days=31, order=	1}, -- MONTHS[1]
	{name="Feverary",	days=28, order=	2},
	{name="Martch",		days=31, order=	3},
	{name="Avdil",		days=30, order=	4},
	{name="Moi",		days=31, order=	5},
	{name="Jummee",		days=30, order=	6},
	{name="Jullard",	days=31, order=	7},
	{name="Oross",		days=31, order=	8},
	{name="Seldzy",		days=30, order=	9},
	{name="Ocker",		days=31, order=10},
	{name="Nowardy",	days=30, order=11},
	{name="Deckard",	days=31, order=12},
}

YEAR={days=0,ticks=0}
--calculate length of year
for i, v in ipairs(MONTHS) do
	YEAR.days=YEAR.days+v.days
end
YEAR.ticks=YEAR.days*10000000*3600*24*YEAR.days


SEASONS = {
-- February 21 to April 30
{name="Watersdays"	,first={2,21}},
-- May 1 to June 20
{name="Bloomdays"	,first={5,1}},
-- June 21 to August 20
{name="Heatdays"	,first={6,21}},
-- August 21 to October 20
{name="Goodtime"	,first={8,21}},
-- October 21 to December 21
{name="Cooldown"	,first={10,21}},
-- December 1 to February 20
{name="Darktime"	,first={12,1}}
}




